from pathlib import Path, PurePosixPath
import pandas as pd
from typing import List, Tuple
from rich import print
from sleepyconvert_toolchain.params import *


def readData(format:str, path:str) -> pd.DataFrame:
  '''dispatch table to read data file according to file extension'''
  return {
    'csv':pd.read_csv,
    'json':pd.read_json,
    'parquet':pd.read_parquet,
    'pkl':pd.read_pickle,
    'xlsx':pd.read_excel,
  }[format](path)


def writeData(df:pd.DataFrame, format:str, path:str, compress:bool) -> None:
  '''dispatch table to write data file according to file extension'''
  if compress:
    path = path + '.gz'
  {
    'csv':df.to_csv,
    'json':df.to_json,
    'parquet':df.to_parquet,
    'pkl':df.to_pickle,
    'xlsx':df.to_excel,
  }[format](path, compression='gzip' if compress else None)


def verifyPaths(input_path:str, output_path:str, supported_formats:List[str]) -> Tuple[str, str]:
  '''check if input and output paths are valid, pass back formats'''
  path_object = Path(input_path)
  input_format = PurePosixPath(input_path).suffix.lower()[1:]
  output_format = PurePosixPath(output_path).suffix.lower()[1:]
  if not path_object.exists():
    print(f'Error. Path {input_path} does not exist.')
    return '', ''
  elif not path_object.is_file():
    print(f'Error. Path {input_path} is not a file.')
    return '', ''
  elif input_format not in supported_data_formats:
    print(f'Error. Input format "{input_format}" not supported, must be one of: {", ".join(supported_data_formats)}')
    return '', ''
  elif output_format not in supported_data_formats:
    print(f'Error. Output format "{output_format}" not supported, must be one of: {", ".join(supported_data_formats)}')
    return '', ''
  return input_format, output_format



from sys import exit
from rich import print
import typer
from sleepyconvert_toolchain.params import *
from sleepyconvert_toolchain.utils import *

app = typer.Typer(help="Convert common filetypes and data formats quickly.")


@app.command()
def data(input_path:str, output_path:str, compress:bool=False):
  '''Convert data file from one format to another, optionally compressing output.'''
  input_format, output_format = verifyPaths(input_path, output_path, supported_data_formats)
  if not (input_format and output_format):
    exit(1)
  # additional guards
  if input_format == output_format:
    print(f'Input and output formats are the same: {input_format}')
    exit(1)
  if compress and output_format == 'xlsx':
    print(f'Error. Cannot compress xlsx files. Please remove the --compress flag.')
    exit(1)

  # read
  try:
    df = readData(input_format, input_path)
  except Exception as e:
    print(f'Error reading data:\n{e}')

  # write
  try:
    writeData(df, output_format, output_path, compress)
  except Exception as e:
    print(f'Error writing data:\n{e}')

  # log
  output_path_display = f'{output_path}.gz' if compress else output_path
  print(f'✅ Converted [bold green]{input_path}[/bold green] ──▶ [bold green]{output_path_display}[/bold green]')


@app.command()
def img(input_path:str, output_path:str, compressed:bool=False):
  '''Convert image file from one format to another'''
  input_format, output_format = verifyPaths(input_path, output_path, supported_img_formats)
  if not (input_format and output_format):
    exit(1)

  # read
  try:
    pass # TODO: parse image into object
  except Exception as e:
    print(f'Error reading image:\n{e}')

  # write
  try:
    pass # TODO: write image object to file
  except Exception as e:
    print(f'Error writing image:\n{e}')


@app.command()
def doc(input_path:str, output_path:str):
  '''Convert document file from one format to another'''
  input_format, output_format = verifyPaths(input_path, output_path, supported_doc_formats)
  if not (input_format and output_format):
    exit(1)

  # read
  try:
    pass # TODO: parse image into object
  except Exception as e:
    print(f'Error reading image:\n{e}')

  # write
  try:
    pass # TODO: write image object to file
  except Exception as e:
    print(f'Error writing image:\n{e}')



supported_data_formats = [
  'csv',
  'json',
  'parquet',
  'pkl',
  'xlsx',
]

supported_img_formats = [] # TODO: add supported image formats

supported_doc_formats = [] # TODO: add supported document formats